#ifndef USBjoystick_h_
#define USBjoystick_h_

#if defined(USB_HID)

#include <inttypes.h>

// C language implementation
#ifdef __cplusplus
extern "C" {
#endif
int usb_joystick_send(void);
extern uint32_t usb_joystick_data[4];
#ifdef __cplusplus
}
#endif

// 32 bits: buttons

// 30 bits: x, y, z
//  2 bits: unused

// 30 bits: rx, ry, rz
//  2 bits: unused

// 20 bits: slider1, slider2
//  4 bits: hat switch


// C++ interface
#ifdef __cplusplus
class usb_joystick_class
{
        public:
        void begin(void) { }
        void end(void) { }
	void button(uint8_t button, bool val) {
		if (--button >= 32) return;
		if (val) usb_joystick_data[0] |= (1 << button);
		else usb_joystick_data[0] &= ~(1 << button);
		if (!manual_mode) send_now();
	}
	void X(unsigned int val) {
		if (val > 1023) val = 1023;
		usb_joystick_data[1] = (usb_joystick_data[1] & 0x3FFFFC00) | val;
		if (!manual_mode) send_now();
	}
	void Y(unsigned int val) {
		if (val > 1023) val = 1023;
		usb_joystick_data[1] = (usb_joystick_data[1] & 0x3FF003FF) | (val << 10);
		if (!manual_mode) send_now();
	}
	void position(unsigned int x, unsigned int y) {
		if (x > 1023) x = 1023;
		if (y > 1023) y = 1023;
		usb_joystick_data[1] = (usb_joystick_data[1] & 0x3FF00000)
			| x | (y << 10);
		if (!manual_mode) send_now();
	}
	void Z(unsigned int val) {
		if (val > 1023) val = 1023;
		usb_joystick_data[1] = (usb_joystick_data[1] & 0x000FFFFF) | (val << 20);
		if (!manual_mode) send_now();
	}
	void Xrotate(unsigned int val) {
		if (val > 1023) val = 1023;
		usb_joystick_data[2] = (usb_joystick_data[2] & 0x3FFFFC00) | val;
		if (!manual_mode) send_now();
	}
	void Yrotate(unsigned int val) {
		if (val > 1023) val = 1023;
		usb_joystick_data[2] = (usb_joystick_data[2] & 0x3FF003FF) | (val << 10);
		if (!manual_mode) send_now();
	}
	void Zrotate(unsigned int val) {
		if (val > 1023) val = 1023;
		usb_joystick_data[2] = (usb_joystick_data[2] & 0x000FFFFF) | (val << 20);
		if (!manual_mode) send_now();
	}
	void sliderLeft(unsigned int val) {
		if (val > 1023) val = 1023;
		usb_joystick_data[3] = (usb_joystick_data[3] & 0x00FFFC00) | val;
		if (!manual_mode) send_now();
	}
	void sliderRight(unsigned int val) {
		if (val > 1023) val = 1023;
		usb_joystick_data[3] = (usb_joystick_data[3] & 0x00F003FF) | (val << 10);
		if (!manual_mode) send_now();
	}
	void slider(unsigned int val) {
		if (val > 1023) val = 1023;
		usb_joystick_data[3] = (usb_joystick_data[3] & 0x00F00000)
			| val | (val << 10);
		if (!manual_mode) send_now();
	}
        inline void hat(int dir) {
                uint32_t val;
                if (dir < 0) val = 15;
                else if (dir < 23) val = 0;
                else if (dir < 68) val = 1;
                else if (dir < 113) val = 2;
                else if (dir < 158) val = 3;
                else if (dir < 203) val = 4;
                else if (dir < 245) val = 5;
                else if (dir < 293) val = 6;
                else if (dir < 338) val = 7;
		usb_joystick_data[3] = (usb_joystick_data[3] & 0x000FFFFF) | (val << 20);
                if (!manual_mode) send_now();
        }
	void useManualSend(bool mode) {
		manual_mode = mode;
	}
	void send_now(void) {
		usb_joystick_send();
	}
	private:
	static uint8_t manual_mode;
};
extern usb_joystick_class Joystick;

#endif // __cplusplus

#endif // USB_HID
#endif // USBjoystick_h_

